( cd usr/lib64 ; rm -rf libvkd3d-shader.so )
( cd usr/lib64 ; ln -sf libvkd3d-shader.so.1.0.0 libvkd3d-shader.so )
( cd usr/lib64 ; rm -rf libvkd3d-shader.so.1 )
( cd usr/lib64 ; ln -sf libvkd3d-shader.so.1.0.0 libvkd3d-shader.so.1 )
( cd usr/lib64 ; rm -rf libvkd3d-utils.so )
( cd usr/lib64 ; ln -sf libvkd3d-utils.so.1.1.0 libvkd3d-utils.so )
( cd usr/lib64 ; rm -rf libvkd3d-utils.so.1 )
( cd usr/lib64 ; ln -sf libvkd3d-utils.so.1.1.0 libvkd3d-utils.so.1 )
( cd usr/lib64 ; rm -rf libvkd3d.so )
( cd usr/lib64 ; ln -sf libvkd3d.so.1.2.0 libvkd3d.so )
( cd usr/lib64 ; rm -rf libvkd3d.so.1 )
( cd usr/lib64 ; ln -sf libvkd3d.so.1.2.0 libvkd3d.so.1 )
